from django.conf.urls.static import static
from django.contrib import admin
from django.urls import path, include, re_path
from django.views.generic import TemplateView
from rest_framework.schemas import get_schema_view

from netguru import settings
from shares import views as shares_views, api as shares_api
from counting import views as counting_views

urlpatterns = [
                  path('accounts/', include('django.contrib.auth.urls')),
                  path('accounts/profile', shares_views.add_share),
                  path('api/add', shares_api.AddShare.as_view()),
                  path('api/history', counting_views.GetHistory.as_view()),
                  path('api/get/<int:share_id>', shares_api.GetShare.as_view()),
                  re_path(r'shares/(?P<share_id>[0-9]+)$', shares_views.view_share),
                  path('admin/', admin.site.urls),
                  path('', shares_views.add_share),
              ] + static('/static/', document_root='/app/media/')

if settings.DEBUG:
    urlpatterns += [
        path('swagger-ui/', TemplateView.as_view(
            template_name='swagger-ui.html',
            extra_context={'schema_url': 'openapi-schema'}
        ), name='swagger-ui'),
        path('openapi', get_schema_view(
            title="Netguru recruitment task",
            version="1.0"
        ), name='openapi-schema'),
    ]
