#
# APOLOGIES IN ADVANCE!!!
#
# I realize that I would be best practice to deduplicate some code contained here
# but since I'm running about 1,5 of full-time equvialents you just have to forgive me
# for sparing the effort to do so. Thanks "from the mountain!"
#

from django.contrib.auth.models import User
from django.test import TestCase, Client
from django.test.html import parse_html
from satella.files import write_to_file


def find_element(document, field, value):
    if isinstance(document, str):
        return
    attrs = dict(document.attributes)
    if attrs.get(field) == value:
        document.attributes = attrs
        return document
    for child in document.children:
        a = find_element(child, field, value)
        if a:
            return a


class TestShares(TestCase):

    def setUp(self) -> None:
        super().setUp()
        user, _ = User.objects.get_or_create(username='testuser', email='admin@admin.com',
                                             password='12345')
        self.client = Client()  # May be you have missed this line
        self.client.force_login(user)

    def test_add_file(self):
        """
        Test that a file can be added and that it can be visited
        """
        FILE_CONTENTS = b'somedata'
        write_to_file('test', FILE_CONTENTS)
        response = self.client.get('http://127.0.0.1/accounts/profile')

        self.assertEqual(response.status_code, 200)
        html = parse_html(response.content.decode('utf-8'))
        csrf_token = find_element(html, 'name', 'csrfmiddlewaretoken').attributes['value']
        with open('test', 'rb') as f_in:
            response = self.client.post('http://127.0.0.1/accounts/profile', {
                'file': f_in,
                'csrfmiddlewaretoken': csrf_token
            })
        self.assertEqual(response.status_code, 200)

        html = parse_html(response.content.decode('utf-8'))
        link = find_element(html, 'id', 'link_url').children[0].replace('https', 'http')
        password = find_element(html, 'id', 'link_password').children[0]

        response = self.client.get(link)
        self.assertEqual(response.status_code, 200)
        html = parse_html(response.content.decode('utf-8'))
        csrf_token = find_element(html, 'name', 'csrfmiddlewaretoken').attributes['value']
        response = self.client.post(link, data={
            'password': password,
            'csrfmiddlewaretoken': csrf_token
        })
        resp = response.streaming_content
        self.assertEqual(b''.join(resp), FILE_CONTENTS)
        self.assertEqual(response.status_code, 200)

    def test_add_url(self):
        """
        Test that an URL can be added and that it can be visited
        """
        response = self.client.get('http://127.0.0.1/accounts/profile')

        self.assertEqual(response.status_code, 200)
        html = parse_html(response.content.decode('utf-8'))
        csrf_token = find_element(html, 'name', 'csrfmiddlewaretoken').attributes['value']

        response = self.client.post('http://127.0.0.1/accounts/profile', {
            'url': 'https://example.com',
            'csrfmiddlewaretoken': csrf_token
        })
        self.assertEqual(response.status_code, 200)

        html = parse_html(response.content.decode('utf-8'))
        link = find_element(html, 'id', 'link_url').children[0].replace('https', 'http')
        password = find_element(html, 'id', 'link_password').children[0]

        response = self.client.get(link)
        self.assertEqual(response.status_code, 200)
        html = parse_html(response.content.decode('utf-8'))
        csrf_token = find_element(html, 'name', 'csrfmiddlewaretoken').attributes['value']
        response = self.client.post(link, {
            'password': password,
            'csrfmiddlewaretoken': csrf_token
        })
        self.assertEqual(response.status_code, 302)
